class Translated:
    def __init__(self, original=None, translated=None, pronounce_original=None, pronounce_translated=None):
        self.original = original
        self.translated = translated
        self.pronounce_original = pronounce_original
        self.pronounce_translated = pronounce_translated



from __future__ import annotations

import re
import logging
from typing import List, Tuple, Iterator
from translatesubs.translators.itranslator import ITranslator
from translatesubs.translators.language import Language
from translatesubs.utils.constants import ENDS_OF_SENTENCES, DEFAULT_SEPS, SEP_MAX_LENGTH


class LanguageManager:

    def __init__(self, to_lang: Language, ignore_line_ends: bool, translator: ITranslator):
        self.to_lang = to_lang
        # Separator was chosen after noting that not all will be treated as non-text object and others
        # such as @@ will not translate all words e.g. "Connect  @@  Something else..." Will not translate correctly :/
        # While Connect  ##  Something else... will do.
        self.separator = DEFAULT_SEPS[0]
        self.ignore_line_ends = ignore_line_ends
        self.translator = translator
        self.prepared = None

    @classmethod
    def create_instance(cls, to_lang: str, ignore_line_ends: bool, translator: ITranslator) -> LanguageManager:
        language = translator.detect_language(to_lang)
        return cls(language, ignore_line_ends, translator) if language else None

    def set_separator(self, new_separator: str):
        self.separator = new_separator

    def prep_for_trans(self, text: Iterator[str]):
        self.prepared = self._prepare_for_translation(text)

    def translate_text(self, pronounce_origin: bool, pronounce_trans: bool) -> Tuple[List[str], List[str]]:
        if not self.prepared:
            raise Exception('Text needs to be prepared for translation first.')

        properly_separated_chunks = self.combine_with_separator()
        translated = self.translator.translate(properly_separated_chunks, self.to_lang.abbreviation)

        # Noticed that when separator contains spaces e.g. ' ∞ ', translated to certain languages separator gets
        # modified e.g. English to Japanese "Hello ∞ everyone" -> "みなさん、こんにちは∞" OR "Minasan, kon'nichiwa ∞"
        sep = self.separator.strip()

        extracted_original = []
        extracted_translated = []

        for trans in translated:
            extracted_original.extend(LanguageManager._extract_translation(
                trans.pronounce_original if pronounce_origin else trans.original, sep))

            extracted_translated.extend(LanguageManager._extract_translation(
                trans.pronounce_translated if pronounce_trans else trans.translated, sep))

        return extracted_original, extracted_translated

    @staticmethod
    def valid_translation(original, translated):
        valid = original and translated and len(original) == len(translated)
        if not valid:
            print(f'original length={len(original)}, translated length={len(translated)}')
        return valid

    @staticmethod
    def _extract_translation(chunk, separator):
        return [sub.strip() for sub in chunk.split(separator)]

    def _prepare_for_translation(self, text_lst: Iterator[str]) -> List[List[str]]:
        """Prepares list of text to be translated by going through every text element in the list and
        grouping them into allowed char 5000 limits, which is placed by google translate service.
        _next_available_sentence function generates a list of text, that comprise a full sentence. Then
        sentences are grouped into larger list, which ensures that if SEP_MAX_LENGTH line separator is added, the total
        character count doesn't take up more than the allowed 5000 limit.

        If this is not done, too many requests will be generated and Google will block the translate service. It should
        also improve the accuracy, since Translator has the access to the whole sentence rather than just a part of it.

        Example:
        The bottom list of text using char limit of 40 will be require going sentence by sentence
        (using _next_available_sentence) and checking if new sentence fits within char limits:
        ["This is an,", "example text!", "I am writing this now..."] ->
        new_sentence = ["This is an,", "example text!"] ->
        24 chars in new_sentence, 0 + 24 = 24 chars in total inside this part, thus save it in:
        single_chunk.extend(new_sentence)

        Get a new sentence and check if that still fits:
        new_sentence = ["I am writing this now..."] ->
        24 chars in new_sentence, 24 + 24 = 48 chars in total - TOO MUCH! Save previous part and store this in new:
        chunks.append(new_sentence)
        single_chunk = []
        single_chunk.extend(new_sentence)

        This will finally generate such a nested list:
        [["This is an,", "example text!"], ["I am writing this now..."]]

        This is overall slower than the regex solution for large chunk size"""
        grouped_chunks = []
        single_chunk = []
        char_count = 0
        separator_length = SEP_MAX_LENGTH
        for sentence in self._next_available_sentence(text_lst):
            sentence_length = sum([len(part) for part in sentence]) + len(sentence) * separator_length
            logging.debug(f'sentence: {sentence[0][:10]}...{sentence[-1][-10:]} with {len(sentence)} texts, '
                          f'{char_count} (char count) + {sentence_length} (sentence length) '
                          f'= {char_count + sentence_length} (total)')
            if char_count + sentence_length > self.translator.get_char_limit():
                logging.debug(f'Reached the {self.translator.get_char_limit} char limit!')
                grouped_chunks.append(single_chunk)
                char_count = 0
                single_chunk = []

            char_count += sentence_length
            single_chunk.extend(sentence)

        if single_chunk:
            grouped_chunks.append(single_chunk)

        return grouped_chunks

    def combine_with_separator(self):
        # Combine text chunks by some GOOD separator, such as ' ## ' that google translate would keep in place instead
        # of removing after translation. This will allow us to send all of the text to be sent for translation,
        # yet still track of the what lines belong to which timestamp
        chunks_to_translate = [self.separator.join(partial) for partial in self.prepared]
        logging.debug(f'Prepared {len(chunks_to_translate)} chunks to be translated.')
        return chunks_to_translate

    @staticmethod
    def _next_available_sentence(text_lst: Iterator[str]) -> Iterator[str]:
        """Generates a list of text, that comprise a full sentence."""
        all_possible_endings = ''.join(ENDS_OF_SENTENCES.values())
        sentence_endings = re.compile(f'[{all_possible_endings}]$', flags=re.DOTALL)
        collected = []

        for sentence in text_lst:
            collected.append(sentence)
            if sentence_endings.search(sentence):
                yield collected
                collected = []

        if collected:
            yield collected
        return

    def _prepare_for_translation_using_regex(self, text_lst: Iterator[str]) -> List[str]:
        """Prepares list of text to be translated by firstly joining the lines using a special separator
        and then taking a chunk of max allowed char and looking back for matching sentence end.

        E.g. the bottom list of text using char limit of 40 will be firstly joined into one string:
        ["This is an,", "example text!", "I am writing this now..."] ->
        "This is an, ## example text! ## I am writing this now..."

        Then it will be separated into parts:
        "This is an, ## example text! ## I am wri" AND "ting this now..." -> (take first 40 chars)
        "This is an, ## example text!" AND " ## I am writing this now..." -> (use regex to find last full sentence)
        ["This is an, ## example text!", "I am writing this now..."] -> (repeat for the second part until done)
        """
        # Slower method
        # .+[.!?"\')]( ## ).+$
        # Quicker method
        # TODO check if need to use * or + at the end: *$ vs +$, original was +$
        # [.!?"\')]( ## )(?:.(?![.!?"\')] ## ))*$
        # OR when no valid endings found e.g. some strange language or subs don't have sentence endings incorporated..
        # ( ## )(?:.(?! ## ))*$
        full_text = self.separator.join(text_lst)
        valid_endings = self._determine_valid_endings(full_text)

        # special regex characters need to be escaped. For now $ works very well but in regex it also matches the end
        # of sentence, thus escape it :)
        processed_separator = self.separator.replace('$', '\\$')
        # Note that when no valid endings found, simply separate by the closest subtitle instead
        splitter = re.compile(f'{valid_endings}({processed_separator})'
                              f'(?:.(?!{valid_endings}{processed_separator}))*$', flags=re.DOTALL)

        curr_index = 0
        chunks_to_translate = []
        total_length = len(full_text)

        while curr_index + self.translator.get_char_limit() < total_length:
            curr_split = full_text[curr_index:curr_index + self.translator.get_char_limit()]
            split_place = splitter.search(curr_split)
            if not split_place:
                exit('It seems like some translations got corrupted. Try a different separator using --separator '
                     'argument. Check --help menu for more information.')
            chunks_to_translate.append(curr_split[:split_place.start(1)])
            curr_index += split_place.end(1)
        chunks_to_translate.append(full_text[curr_index:curr_index + self.translator.get_char_limit()])
        logging.debug(f'Prepared {len(chunks_to_translate)} chunks to be translated.')

        return chunks_to_translate

    def _determine_valid_endings(self, full_text):
        if self.ignore_line_ends:
            return ''
        for name, endings in ENDS_OF_SENTENCES.items():
            re_endings = f'[{endings}]'
            if re.search(re_endings, full_text, flags=re.DOTALL):
                logging.info(f'Using "{name}" endings "{endings}"')
                return re_endings
        logging.info(f'No valid line ends found, thus ignoring them.')
        return ''



import googletrans
from typing import List, Iterator
import logging
import re
from translatesubs.translators.itranslator import ITranslator
from translatesubs.translators.language import Language
from translatesubs.translators.translated import Translated
from translatesubs.utils.tools import nth

"""
The API has problems with pronunciations: 
1. cannot pronounce the original text and
2. cannot pronounce multiple sentences (pronunciation becomes None).

To extract correct pronunciation data, we can look at examples:

1) how are you? -> spanish (origin: cannot pronounce, translation: cannot pronounce)
[['¿cómo estás?', 'how are you?', None, None, 1]]
- if the last element is a number or is not a str, then there are NO translations

2) how are you? -> japanese (origin: cannot pronounce, translation: can pronounce)
[['お元気ですか？', 'how are you?', None, None, 1], [None, None, 'Ogenkidesuka?']]
original pronounced does not exist, translation pronounced [-1]
- if 1) is correct AND length of elements is <4, that means still cannot pronounce original 

3) お元気ですか？ -> english (origin: can pronounce, translation: cannot pronounce)
[['How are you?', 'お元気ですか？', None, None, 1], [None, None, None, 'Ogenkidesuka?']]
original pronounced [-1], translation pronounced is None
- if length >3 and -2 element is None, that means cannot pronounce translation

4) お元気ですか？ -> korean (origin: can pronounce, translation: can pronounce)
[['잘 지내?', 'お元気ですか？', None, None, 0], [None, None, 'jal jinae?', 'Ogenkidesuka?']]
original pronounced [-1], translation pronounced [-2]
- if 1) is correct AND

Logic:
1) If there is a translation AND len == 4, then there is origin language pronunciation at -1 index, else there isn't one
2) If there is a translation AND 2 index is not None, then there is a translation at 2 index, otherwise there isn't one
"""


class GoogleTrans(ITranslator):
    def get_char_limit(self) -> int:
        return 5000

    def translate(self, text: List[str], to_lang: str) -> Iterator[Translated]:
        google_translated = GoogleTrans._do_translate(text, to_lang)
        for original, translated in zip(text, google_translated):
            yield Translated(original=original,
                             translated=translated.text.strip(),
                             pronounce_original=GoogleTrans._pronounce_origin(translated),
                             pronounce_translated=GoogleTrans._pronounce_translated(translated))

    def detect_language(self, to_lang: str) -> Language:
        return next((Language(full, abb) for abb, full in googletrans.LANGUAGES.items()
                     if to_lang == abb or to_lang == full), None)

    def get_supported(self) -> str:
        return ', '.join([f'{abb} - {full}' for abb, full in googletrans.LANGUAGES.items()])

    @staticmethod
    def _do_translate(text: List[str], to_lang: str) -> List[googletrans.models.Translated]:
        """
        Call google translate API to translate given text

        :param **kwargs:
        :param text: The source text(s) to be translated. Batch translation is supported via sequence input.
        :type text: UTF-8 :class:`str`; :class:`unicode`; string sequence (list, tuple, iterator, generator)

        :return: translated text in the same form it was provided
        """
        # Google API provider should allow new access every 1h, but if more translations need to be done,
        # a number of different country providers are given
        # E.g. from here https://sites.google.com/site/tech4teachlearn/googleapps/google-country-codes
        # But have to make sure the site actually loads first :)

        ending_formula = re.compile(r'translate\..*?\.(.+)$')  # for for com, co.uk, lt or others
        provider_endings = (ending_formula.search(url).group(1) for url in googletrans.constants.DEFAULT_SERVICE_URLS)
        provider_base = 'translate.googleapis'

        for ending in provider_endings:
            provider = f'{provider_base}.{ending}'
            translator = googletrans.Translator(service_urls=[provider])
            try:
                return translator.translate(text, dest=to_lang)
            except AttributeError:
                logging.info(f'Provider "{provider}" got blocked, trying another one...')
        exit('No more providers left to try, try updating the provider list or wait 1h until you get unblocked.')

    @staticmethod
    def _pronounce_origin(translated: googletrans.models.Translated) -> str:
        # The only way we have origin pronunciation is when extra_data['translation'] contains at least two items:
        # 1) translation and 2) pronunciation. The latter must then contain 4 items, while the last item is the origin
        # translation. If the pronunciation only contains 3 items, then the 3rd item is the pronounced translation.
        expected = GoogleTrans._expected_pronounced(translated)
        if GoogleTrans._can_pronounce(expected) and GoogleTrans._can_pronounce_original(expected):
            return expected[-1]
        return translated.origin

    @staticmethod
    def _pronounce_translated(translated: googletrans.models.Translated) -> str:
        # When returning pronunciation of translated, the API returns pronunciation whenever one is available.
        # If however, it is not available it returns original text, while I would want it to return the translated text
        # instead, since that simply means that the text is already pronounced.
        expected = GoogleTrans._expected_pronounced(translated)
        if GoogleTrans._can_pronounce(expected) and GoogleTrans._can_pronounce_translated(expected):
            return expected[2]
        return translated.text

    @staticmethod
    def _expected_pronounced(translated: googletrans.models.Translated) -> List:
        return nth(translated.extra_data['translation'], -1)

    @staticmethod
    def _can_pronounce(pronounced: List) -> bool:
        return isinstance(nth(pronounced, -1), str)

    @staticmethod
    def _can_pronounce_original(pronounced: List) -> bool:
        return len(pronounced) == 4

    @staticmethod
    def _can_pronounce_translated(pronounced: List) -> bool:
        return pronounced[2] is not None



#!/usr/bin/env python

from translatesubs.managers.language_manager import LanguageManager
from translatesubs.managers.subs_manager import SubsManager
from translatesubs.utils.constants import AVAILABLE_TRANSLATORS, TRANSLATORS_PRINT, DEFAULT_SEPS_PRINT, USE_DEFAULT_SEPS, \
    DEFAULT_SEPS, SEP_MAX_LENGTH, SUB_FORMATS

import argparse
import logging
import sys
from typing import List
import os

"""
Future Development:
1) Automatically detect the subs language using "from_lang" argument, which would be more automatic than subs_track.
2) Add official google translate api and ability to setup your google api key.
"""


def main():
    parser = argparse.ArgumentParser(
        description='It is a tool to translate movie subtitles from one language into another, or even show multiple '
                    'language subtitles together.',
        usage='python translatesubs.py video.mkv output.ass --merge --to_lang fr')
    parser.add_argument('input', type=str,
                        help='Input file to translate; By default it is a subtitle file but if flag --video_file is'
                             ' set, then this is video file name.')
    parser.add_argument('output', type=str, help='Generated translated subtitle file.')
    parser.add_argument('--encoding', default='utf-8', type=str,
                        help='Input file encoding, which defaults to "utf-8". To determine it automatically use "auto"')
    parser.add_argument('--to_lang', default='es', type=str, help='Language to which translate to.')
    parser.add_argument('--pronounce_original', action='store_true',
                        help='Use pronunciation rather than writing form for origin subs e.g. useful for Japanese')
    parser.add_argument('--pronounce_translated', action='store_true',
                        help='Use pronunciation rather than writing form for translated subs e.g. useful for Japanese')
    parser.add_argument('--merge', action='store_true',
                        help='Set to see both, the original (at bottom) and the translated (on top), subtitles.')
    parser.add_argument('--reverse', action='store_true',
                        help='Display original subs on top and translated at the bottom instead, when --merge is set.')
    parser.add_argument('--secondary_scale', default=80, type=int,
                        help='Specify the secondary subs scale factor where 100 is its original size.')
    parser.add_argument('--line_char_limit', default=30, type=int,
                        help='Decide if keep multiple, often short, lines or merge them into one instead. Best '
                             'used with --merge flag since then extra lines are added. Recommended value 30 or 70.')
    parser.add_argument('--input_type', default='auto', choices=['auto', 'video', 'subs'],
                        help='Specify input file type. By default it tries to automatically deduce the type.')
    parser.add_argument('--subs_track', default=0, type=int,
                        help='Select subtitle track, if video has multiple subtitles attached to it.')
    parser.add_argument('--translator', default='googletrans', type=str,
                        help=f'One of the Translate services to use: {TRANSLATORS_PRINT}. googletrans does a better '
                             'job when pronunciation is needed, since it preserves new lines, however it very easily '
                             'gets corrupted, thus often many different separators have to be tried. On the other hand '
                             'google_trans_new seems to work very well, but removes new line chars in pronunciation...')

    parser.add_argument('--logging', default=40, type=int,
                        help='NOTSET - 0, DEBUG - 10, INFO - 20, WARNING - 30, ERROR - 40, CRITICAL - 50')
    parser.add_argument('--ignore_line_ends', action='store_true',
                        help='Set this when you are sure that subs do not have line end characters such as ?!. or'
                             'others, since we rely on them to split the text correctly! e.g. machine generated subs'
                             'will not include line end characters.')
    parser.add_argument('--separator', default=USE_DEFAULT_SEPS, type=str,
                        help='Special subtitle separator when sending it to be translated. Sometimes will just have to '
                             'experiment, since different language combinations require different one. I found " $$$ " '
                             'to work with majority of languages, however not all of them. The separators are created '
                             'using some weird character in various combinations like "X", " X ", "XX", " XX ", '
                             f'"XXX", " XXX ", where X is that weird character. Do not use more than {SEP_MAX_LENGTH} '
                             'chars as a separator. Separators for these languages work best:\n'
                             'Japanese - " ∞ ", " ™ ", "$$$"\n'
                             'Simplified Chinese - "@@", "@@@"\n'
                             'Albanian - "@@", "@@@"\n'
                             'Polish - "@@@", "$$$", "€€€"\n'
                             'Greek - "$$", " $$ ", "$$$", " $$$ "\n'
                             '...\n'
                             'Be careful when using $ sign, since terminal treats it as argument input, thus need a '
                             'backslash for it.\n'
                             f'Default behavior tries separators one by one from the list: {DEFAULT_SEPS_PRINT}. '
                             'If these do not work, then only some good hack can help u :)')
    args = parser.parse_args()

    logging.basicConfig(stream=sys.stderr, level=args.logging)
    logging.info(f'Using logging level {logging.getLogger()} - lvl {logging.getLogger().level}.')

    # Prepare original subs: extract text and styling
    filename = get_subs_file(args)
    subs_manager = SubsManager(filename=filename, encoding=get_encoding(args.encoding, filename))
    subs_manager.extract_line_styling()

    # Perform translation: prepare extracted subs for translating and try different separators to see which will work
    translator = get_translator(args.translator)
    language_manager = get_language_manager(args.to_lang, args.ignore_line_ends, translator)
    language_manager.prep_for_trans(subs_manager.just_text())
    original, translated = translate(language_manager, separators_to_try(args.separator),
                                     args.pronounce_original, args.pronounce_translated)

    # To display firstly original and translated below instead
    if args.reverse:
        original, translated = translated, original

    subs_manager.update_subs(main_subs=translated, secondary_subs=original,
                             merge=args.merge, secondary_scale=args.secondary_scale,
                             char_limit=args.line_char_limit)
    subs_manager.save_subs(args.output)
    print('Finished!')


def get_encoding(encoding, filename):
    if encoding == 'auto':
        import chardet
        with open(filename, "rb") as f:
            res = chardet.detect(f.read())
        return res['encoding']
    return encoding
    
def get_subs_file(args):
    extension = os.path.splitext(args.input)[1].strip('.')
    if args.input_type == 'subs' or (args.input_type == 'auto' and extension in SUB_FORMATS):
        return args.input

    # must have selected video, simply extract the subtitle and return it's path
    if not SubsManager.extract_from_video(video_in=args.input, subs_track=args.subs_track, subs_out=args.output):
        exit('Could not extract the subtitles!')

    print(f'Extracted subtitles from "{args.input}" into "{args.output}".')
    return args.output


def get_language_manager(to_lang, ignore_line_ends, translator):
    # Ensure that the language is valid and is supported
    language_manager = LanguageManager.create_instance(to_lang=to_lang,
                                                       ignore_line_ends=ignore_line_ends,
                                                       translator=translator)
    if not language_manager:
        exit(f'Cannot detect language "{to_lang}". Supported either abbreviation or full language name:\n'
             f'{translator.get_supported()}.')

    print(f'Translating to "{language_manager.to_lang.full_name}".')
    return language_manager


def get_translator(translator_name):
    # Instantiate one of the translators
    translator = AVAILABLE_TRANSLATORS.get(translator_name, None)
    if not translator:
        exit(f'Translator "{translator_name}" is not supported. '
             f'Try one of the supported ones: {TRANSLATORS_PRINT}.')

    return translator()


def translate(language_manager, separators, pronounce_origin, pronounce_trans):
    for sep in separators:
        print(f'Trying separator "{sep}"...')
        language_manager.set_separator(sep)
        original, translated = language_manager.translate_text(pronounce_origin=pronounce_origin,
                                                               pronounce_trans=pronounce_trans)
        if LanguageManager.valid_translation(original, translated):
            return original, translated

    # This we do not want to reach!
    exit('It seems like all tries to translate got corrupted. Try to manually set the separator using '
         f'--separator argument to be DIFFERENT from: {DEFAULT_SEPS_PRINT}. Check --help menu for more information.')


def separators_to_try(separator_input) -> List[str]:
    if separator_input != USE_DEFAULT_SEPS:
        return [separator_input]

    return DEFAULT_SEPS


if __name__ == "__main__":
    main()



from translatesubs.translators.googletrans import GoogleTrans
from translatesubs.translators.google_trans_new import GoogleTransNew


AVAILABLE_TRANSLATORS = {'googletrans': GoogleTrans,            # Does not keep newlines for pronunciation only
                         'google_trans_new': GoogleTransNew}    # Does not keep newlines
TRANSLATORS_PRINT = ', '.join(AVAILABLE_TRANSLATORS.keys())

ENDS_OF_SENTENCES = {
    'Usual': '.!?"\')',
    'Japanese': 'よねのさぞなか！。」…',
}

USE_DEFAULT_SEPS = 'default'
DEFAULT_SEPS = [' $$$ ', ' ### ', ' ∞ ', '@@@', " ™ ", ' @@@ ', '$$$', '€€€', '££', ' ## ', '@@', '$$']
DEFAULT_SEPS_PRINT = ', '.join((f'\"{sep}\"' for sep in DEFAULT_SEPS))

SEP_MAX_LENGTH = 7

SUB_FORMATS = ('srt', 'ass', 'ssa', 'mpl2', 'tmp', 'vtt', 'microdvd')



def flatten(complex_list):
    return [inner for outer in complex_list for inner in outer]


def nth_only_positive_index(lst, index, default=None):
    return (lst[index:index + 1] + [default])[0]


def nth(lst, index, default=None):
    try:
        return lst[index]
    except IndexError:
        return default



from typing import List, Iterator
from translatesubs.translators.translated import Translated
from translatesubs.translators.language import Language
from abc import ABC, abstractmethod


class ITranslator(ABC):
    @abstractmethod
    def translate(self, text: List[str], to_lang: str) -> Iterator[Translated]:
        pass

    @abstractmethod
    def detect_language(self, to_lang: str) -> Language:
        pass

    @abstractmethod
    def get_supported(self) -> str:
        pass

    @abstractmethod
    def get_char_limit(self) -> int:
        pass



import google_trans_new
from typing import List, Iterator
import logging
import re

from translatesubs.translators.itranslator import ITranslator
from translatesubs.translators.language import Language
from translatesubs.translators.translated import Translated


class GoogleTransNew(ITranslator):
    """
    Google_trans_new suffers from a bug in the API: within the pronunciation, new line characters are removed and
    thus if it is important to preserve perfect styling, you are better off using another translation service.
    """

    def get_char_limit(self) -> int:
        return 5000

    def translate(self, text: List[str], to_lang: str) -> Iterator[Translated]:
        for original in text:
            pronounced = self._do_translate(original, to_lang, pronounce=True)
            translated = self._do_translate(original, to_lang).strip()
            yield Translated(original=original,
                             translated=translated,
                             pronounce_original=GoogleTransNew._pronounce_origin(pronounced, original),
                             pronounce_translated=GoogleTransNew._pronounce_translated(pronounced, translated))

    def detect_language(self, to_lang: str) -> Language:
        return next((Language(full, abb) for abb, full in google_trans_new.LANGUAGES.items()
                     if to_lang == abb or to_lang == full), None)

    def get_supported(self) -> str:
        return ', '.join([f'{abb} - {full}' for abb, full in google_trans_new.LANGUAGES.items()])

    @staticmethod
    def _do_translate(text: str, to_lang: str, pronounce=False):
        """
        Call google translate API to translate given text

        :param **kwargs:
        :param text: The source text(s) to be translated. Batch translation is supported via sequence input.
        :type text: UTF-8 :class:`str`; :class:`unicode`; string sequence (list, tuple, iterator, generator)

        :return: translated text in the same form it was provided
        """
        # Google API provider should allow new access every 1h, but if more translations need to be done,
        # a number of different country providers are given
        # E.g. from here https://sites.google.com/site/tech4teachlearn/googleapps/google-country-codes
        # But have to make sure the site actually loads first :)
        ending_formula = re.compile(r'translate\..*?\.(.+)$')  # for for com, co.uk, lt or others
        provider_endings = (ending_formula.search(url).group(1) for url in google_trans_new.DEFAULT_SERVICE_URLS)

        for ending in provider_endings:
            translator = google_trans_new.google_translator(url_suffix=ending)
            try:
                return translator.translate(text, lang_tgt=to_lang, pronounce=pronounce)
            except AttributeError:
                logging.info(f'Provider "translate.google.{ending}" got blocked, trying another one...')
        exit('No more providers left to try, try updating the provider list or wait 1h until you get unblocked.')

    @staticmethod
    def _pronounce_origin(translated: List[str], default: str) -> str:
        return translated[1] if translated[1] else default

    @staticmethod
    def _pronounce_translated(translated: List[str], default: str) -> str:
        return translated[2] if translated[2] else default



from setuptools import find_packages, setup

with open('README.md', 'r') as fh:
    long_description = fh.read().strip()
    
with open('version.txt', 'r') as fh:
    version = fh.read().strip()

SETUP_ARGS = dict(
    name='translatesubs',
    version=version,
    license='Apache-2.0',
    author='Montvydas Klumbys',
    author_email='montvydas.klumbys@gmail.com',
    description='It is a tool to translate subtitles into any language, that is supported by google translator',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/Montvydas/translatesubs',
    download_url=f'https://github.com/Montvydas/translatesubs/archive/v_{version}.tar.gz',
    keywords=['SUBTITLES', 'TRANSLATE'],
    packages=find_packages(),
    install_requires=[
        'pysubs2==1.6.0',
        'googletrans==3.1.0a0',
        'google-trans-new==1.1.9',
        'chardet==3.0.4',
        'requests==2.28.2',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License ',
        'Operating System :: OS Independent',
        'Topic :: Multimedia :: Video',
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'translatesubs=translatesubs.main:main'
        ]
    },
)


if __name__ == "__main__":
    setup(**SETUP_ARGS)



import pysubs2
import subprocess
import logging
from typing import List, Iterator
import re


class Sub:
    def __init__(self, text: str, plaintext: str):
        self.origin_text = text
        self.plaintext = plaintext
        self.open_style = ''
        self.close_style = ''

    @staticmethod
    def to_plaintext(sub: pysubs2.SSAEvent):
        return sub.plaintext if sub.plaintext else sub.text

    @staticmethod
    def merge_multiline(multiline: str, char_limit: int):
        lines_length = len(multiline.replace('\n', ''))
        return multiline.replace('\n', ' ').replace(' ,', ',') if lines_length < char_limit else multiline

    def extract_line_styling(self):
        match = re.search(r'^{.+?}', self.origin_text, flags=re.DOTALL)
        if match:
            # logging.info(f'Opening: {match.group()}')
            self.open_style = match.group()

        match = re.search(r'.+({.+?})$', self.origin_text, flags=re.DOTALL)
        if match:
            # logging.info(f'Closing: {match.group(1)}')
            self.close_style = match.group(1)


class SubsManager:
    def __init__(self, filename: str, encoding: str='utf-8'):
        try:
            self.origin_subs = pysubs2.load(filename, encoding)
        except UnicodeDecodeError as e:
            exit(f'{e}\nTry changing encoding manually or allow "chardet" lib to determine it with: --encoding auto')
        self.subs = [Sub(sub.text, Sub.to_plaintext(sub)) for sub in self.origin_subs]

    def extract_line_styling(self):
        logging.info('Extracting individual line styling..')
        [sub.extract_line_styling() for sub in self.subs]

    def just_text(self) -> Iterator[str]:
        return (sub.plaintext for sub in self.subs)

    def update_subs(self, main_subs: List[str], secondary_subs: List[str], merge: bool, secondary_scale: int, char_limit: int):
        # original --> secondary
        # translated --> main
        for main, secondary, sub, origin_sub in zip(main_subs, secondary_subs, self.subs, self.origin_subs):
            # 1. For now ignore the in-line based styling e.g. bold single word.
            # 2. Replace \n with \N as otherwise the same sub will be treated as separate event aka next sub.
            # NOTE: When writing into plaintext, \n is replaced with \N. But we also want to add custom styling..            
            main = Sub.merge_multiline(main, int(char_limit))
            main = SubsManager._replace_with_capital_newline(main)
            
            if merge:
                secondary = Sub.merge_multiline(secondary, int(char_limit * 100 / secondary_scale))
                secondary = SubsManager._replace_with_capital_newline(secondary)
                secondary = SubsManager._style_secondary(secondary, secondary_scale)
            else:
                secondary = ""

            origin_sub.text = f'{sub.open_style}{main}{secondary}{sub.close_style}'

    def save_subs(self, subs_out: str):
        self.origin_subs.save(subs_out)

    @staticmethod
    def extract_from_video(video_in: str, subs_track: int, subs_out: str) -> bool:
        operation = ['ffmpeg', '-i', video_in, '-map', f'0:s:{subs_track}', subs_out]
        logging.debug(f'Extracting subs using {" ".join(operation)}')
        status = subprocess.run(operation)
        return status.returncode == 0

    @staticmethod
    def _style_secondary(text: str, scale: int) -> str:
        # Make text smaller than original and add 50% transparency - note it's HEX, not decimal.
        open_style = f'\\N{{\\fscx{scale}\\fscy{scale}\\alpha&H75&}}'
        close_style = '\\N{\\fscx100\\fscy100\\alpha&H00&}'
        return f'{open_style}{text}{close_style}'

    @staticmethod
    def _replace_with_capital_newline(multiline):
        return multiline.replace('\n', '\\N')
